# -*- coding: utf-8 -*-

from guietta import B, _, Gui, Quit


gui = Gui(
    
  [  'Enter expression:', '__expr__'  , B('Eval!') ],
  [  'Result:'          , 'result'    , _          ],
  [  _                  , _           , Quit       ] )

gui.run()


    


# -*- coding: utf-8 -*-

from guietta import B, E, HS, _, Gui, Quit



gui = Gui(
    
  [  'Enter expression:', E('expr')  , B('Eval!') ],
  [  'Result:'          , 'result'    , _          ],
  [  HS('myslider')      , _        , Quit       ] )


while True:
    name, event = gui.get()

    if name == 'Eval':
        gui.result = eval(gui.expr)
    
    if name == 'myslider':
        gui.result = event.args[0]

    elif name == None:
        break


# -*- coding: utf-8 -*-


import numpy as np
from guietta import Gui, PG, PGI, III, VS, Exceptions

gui = Gui(
  [  PG('plot'),  VS('slider') ],
  [     III     ,  '^^^ Move the slider'  ],
  [ PGI('image') ,  '<< image'],
  exceptions =  Exceptions.OFF
    )

with gui.slider:
    t = np.linspace(0, 1+gui.slider/10, 400)
    gui.plot = np.tan(t)

    img = t.reshape((20,20))
    gui.image = img

gui.slider = 1

gui.run()



# -*- coding: utf-8 -*-

from guietta import E, _, Gui, Quit

gui = Gui(
    
  [  'Enter expression:', E('expr')  , _ ],
  [  'Result:'          , 'result'    , _          ],
  [  _                  , _           , Quit       ] )


gui.events(
    
    [  _            ,   ('textEdited', None) , _    ], 
    [  _            ,   _                       , _    ], 
    [  _            ,   _                       , _    ], )


while True:
    name, event = gui.get()

    if name == 'expr':
        try:
            gui.result = eval(gui.expr)
        except Exception as e:
            gui.result = 'Error: ' + str(e)

    elif name == None:
        break
    


# -*- coding: utf-8 -*-

import os.path
from guietta import B, L, _, Gui

gui = Gui(
    
  [  _             , L('up.png')   , _              ],
  [  B('left.png') ,     _         , B('right.png') ],
  [  _             , L('down.png') , _           ,  ],

  images_dir = os.path.dirname(__file__))

while True:
    name, event = gui.get()

    print(name, event)

    if name == None:
        break





# -*- coding: utf-8 -*-

from guietta import Gui, QMessageBox, PW

gui = Gui(
  [  'Enter password:', PW('password') , ['Check'] ]
)

# Accept both the Enter key and a click on "Check"

with gui.Check, gui.password:
    if gui.is_running:
        if gui.password == '123456':
            QMessageBox.information(None, "Correct!", 'Correct!')
        else:
            QMessageBox.information(None, "Wrong!", 'Wrong!')

gui.run()




# -*- coding: utf-8 -*-

import numpy as np
from guietta import Gui, ___, III, _, M, Ax, VS

def replot(gui, value):

    with Ax(gui.plot) as ax:
        ax.set_title('y=tan(x)')
        t = np.linspace(0, 1+value/10, 500)
        ax.plot(t, np.tan(t), ".-")


def click(gui, x, y):
    print('Mouse click:', x, y)

gui = Gui(
    
  [  M('plot'), ___, ___, VS('slider') ],
  [     III     , III, III,     III      ],
  [     III     , III, III,     III      ],
  [     III     , III, III,  '^^^ Move the slider'  ],
 )

gui.events(
    
    [  click        ,  _ , _ ,   replot    ], 
    [  _            ,  _ , _ ,   _          ], 
    [  _            ,  _ , _ ,   _          ], )


replot(gui, 1)

gui.run()

    


# -*- coding: utf-8 -*-

from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import FigureCanvas

from guietta import Signal, _alsoAcceptAnotherGui, Ax


class MatplotlibWidget(FigureCanvas):

    clicked = Signal(float, float)

    def __init__(self, width, height, dpi, subplots, animated=False, projection=None, **kwargs):
        figure = Figure(figsize=(width, height), dpi=dpi)
        # DO not use add_subplots(), for compatibility with
        # old versions of maplotlib (<2.1)
        if subplots == (1, 1):
            self.ax = figure.add_subplot(1, 1, 1, projection=projection)
        else:
            self.ax = []
            for x in range(subplots[0] * subplots[1]):
                self.ax.append(figure.add_subplot(subplots[0], subplots[1],
                                                  x+1, projection=projection))
        self.kwargs = kwargs
        self.animated = animated
        self.plotobj = None
        super().__init__(figure)
        figure.canvas.mpl_connect('button_press_event',
                                  self._on_button_press)

    def image(self):
        images = self.ax.get_images()
        if len(images) > 0:
            return images[0]
        else:
            return None

    def colorbar(self, *args, **kwargs):
        image = self.image()
        if image:
            self.ax.get_figure().colorbar(image, *args, **kwargs)
        else:
            raise Exception('There is no image to attach the colorbar to')

    def __getattr__(self, name):
        if hasattr(self.ax, name):
            return getattr(self.ax, name)
        elif hasattr(self.image(), name):
            return getattr(self.image(), name)
        else:
            raise AttributeError(name)

    def _on_button_press(self, event):
        self.clicked.emit(event.xdata, event.ydata)

    def __guietta_property__(self):

        def getx():
            return self

        @_alsoAcceptAnotherGui(self)
        def setx(x):
            if x is None:
                return

            import numpy as np
            try:
                arr = np.array(x)
            except Exception as e:
                errmsg = 'Matplotlib widgets need an array-like value'
                raise TypeError(errmsg) from e

            if len(arr.shape) not in [1, 2]:
                raise ValueError('Value must be 1d or 2d, shape is %s instead' %
                                 str(arr.shape))

            if not self.animated or self.plotobj is None:
                with Ax(self) as ax:
                    if len(arr.shape) == 1:
                        self.plotobj = ax.plot(arr)
                    elif len(arr.shape) == 2:
                        self.plotobj = ax.imshow(arr)
            else:
                if len(arr.shape) == 1:
                    self.plotobj[0].set_ydata(arr)
                elif len(arr.shape) == 2:
                    self.plotobj.set_array(arr)
                self.ax.figure.canvas.draw()

        return (getx, setx)

# ___oOo___



# -*- coding: utf-8 -*-

import numpy as np
import astropy.units as u
from astropy.constants import G, M_earth, R_earth
from guietta import Gui, ___, HValueSlider

def orbital_speed(h):
    '''h = height over earth surface'''
    return np.sqrt(G * M_earth / (R_earth + h))


hslider = HValueSlider('h', myrange=range(500,40000), unit='km')

gui = Gui(
   ['Orbital height:', hslider, ___ ],
   [ 'results', ___, ___ ],

)

with gui.h:
    info = {}
    info['Orbital height:'] = gui.h * u.km
    info['Orbital speed:'] = orbital_speed(gui.h * u.km)
    info['Orbital length:'] = 2*np.pi*gui.h * u.km
    gui.results = info

gui.run()



# -*- coding: utf-8 -*-

from guietta import Gui, Quit, _

counter=0
def regular_update(gui):
    global counter
    counter += 1
    gui.counter = counter

def myeval(gui):
    gui.result = eval(gui.expr)

def reset(gui):
    global counter
    counter=0

def stop(gui):
    gui.timer_stop()

def start(gui):
    gui.timer_start(regular_update, interval=0.1)


gui = Gui(
    
  [  'Enter expression:', '__expr__'  , ['Eval!'] ],
  [  'Result:'          , 'result'    , _         ],
  [  ['Start']          , ['Stop']    , ['Reset'] ],
  [  'counter'          , _           , Quit      ] )

gui.events(
    
  [        _            ,    myeval   ,   myeval   ],
  [        _            ,    _        ,    _       ],
  [     start           ,   stop      ,   reset    ],
  [        _            ,    _        ,    _       ],  )

start(gui)
gui.run()


    




# -*- coding: utf-8 -*-

import sys
from guietta import Gui, ___, III

if len(sys.argv) < 2:

    gui = Gui(
        
      [ 'Big label', ___  , 'xxx'],
      [     III    , III  , 'xxx'],
      [    'xxx'   ,'xxx'  , 'xxx'],
      )

elif sys.argv[1] == 'row':

     # This one will cause an exception because of ___ at the beginnng
     # of a row
     
     gui = Gui(
        
         [ 'xxx' , 'xxx' , 'xxx'],
         [  ___  ,  III  , 'xxx'],
         [ 'xxx' , 'xxx' , 'xxx'],
    )

elif sys.argv[1] == 'col':

     # This one will cause an exception because of III at the beginning
     # of a column
     
     gui = Gui(
        
         [ 'xxx' ,  III  , 'xxx'],
         [ 'xxx' , 'xxx' , 'xxx'],
         [ 'xxx' , 'xxx' , 'xxx'],
     )

else:
    print('Use no arguments or one of "row" or "col"')
    sys.exit(-1)

while True:
    name, event = gui.get()

    if name is None:
        break





'''
Example with different line edit initializations
'''
from guietta import Gui, E, _, Quit, QMessageBox

gui = Gui(
 [ '__foo__'    , _ ],
 [ '__bar__:10' , _ ],
 [ E('tux')     , _ ],
 [ (E('a'), 'b'), _ ],
 [ ('__c__', 'd'),_ ],
 [  ['Show']    , Quit ])

with gui.Show:
    if gui.is_running:
        msg = 'Widget values:\n'
        for widget in 'foo bar tux b d'.split():
            msg += '%s = %s\n' % (widget, gui.proxy(widget).get())
        QMessageBox.information(None, None, msg)

gui.run()




# -*- coding: utf-8 -*-

from guietta import B, _, Gui, Quit

gui = Gui(
    
  [  'Enter expression:', '__expr__'  , B('Eval!') ],
  [  'result'          , 'result'    , _          ],
  [  _                  , _           , Quit       ] )

while True:
    name, event = gui.get()

    if name == 'Eval':
        try:
            gui.result = eval(gui.expr)
        except Exception as e:
            gui.result = 'Error: ' + str(e)

    if name == None:
        break





from guietta import _, Gui, Quit
import guietta

gui = Gui(
	[ "Enter numbers:",  "__a__", "+", "__b__", ["Calculate"] ],
	[    "Result: -->", "result",   _,       _,             _ ],
	[                _,        _,   _,       _,          Quit ]
)

with gui.Calculate:
	if not gui.is_running:
		gui.result = ""
	else:
		gui.result = float(gui.a) + float(gui.b)

gui.run()



# -*- coding: utf-8 -*-

from guietta import _, ___, Gui, Quit, Exceptions
        
gui = Gui(
    
  [  'Enter numbers:', '__num1__' , '+' , '__num2__',   _    ],
  [  'Result:  -->'  , 'result'   , ___ ,  ___      ,   _    ],
  [  _               ,    _       ,  _  ,   _       ,  Quit  ],
  exceptions = Exceptions.POPUP)


# We can remove one of the two widgets in the "with" expression
# and the slot would be executed only when the remaining one is triggered.

with gui.num1, gui.num2:
    gui.result = float(gui.num1) + float(gui.num2)
     
gui.run()



#!/usr/bin/env python
import os
import sys
from shutil import rmtree
import subprocess

from setuptools import setup, Command

NAME = 'guietta'
DESCRIPTION = 'Simple GUI for Python'
URL = 'https://github.com/alfiopuglisi/guietta'
EMAIL = 'alfio.puglisi@inaf.it'
AUTHOR = 'Alfio Puglisi'
LICENSE = 'MIT'
#KEYWORDS = 

# Load the package's __version__.py module as a dictionary.
here = os.path.abspath(os.path.dirname(__file__))
about = {}
with open(os.path.join(here, NAME, '__version__.py'),encoding='utf-8') as f:
    exec(f.read(), about)


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')

        sys.exit()
        
        
class CondaCommand(Command):
    """Build and upload conda package."""

    description = 'Build and publish the package on conda.'
    user_options = []
    
    @staticmethod    
    def using_conda():
        """Check whether inside a conda environment."""
        
        if sys.platform == "win32":
            env_variable = "%CONDA_PREFIX%"
            
        elif sys.platform in ["linux", "darwin"]:
            env_variable = "$CONDA_PREFIX"
            
        command = "echo " + env_variable
        out = subprocess.check_output(command, shell=True).decode('utf-8')
        
        if env_variable in out:
            return False
        else:
            return True

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if not self.using_conda():
            self.status('Checking conda environment status…')
            print("Not inside conda environment.")
            sys.exit()
        
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist_conda'))
        except OSError:
            pass

        self.status('Building conda distribution…')
        os.system('conda build . --output-folder dist_conda/ -c conda-forge')

        self.status('Uploading the package to conda…')
        os.system('anaconda upload dist_conda/noarch/guietta-{}-py_0.tar.bz2'.format(about['__version__']))

        sys.exit()


setup(name=NAME,
      description=DESCRIPTION,
      version=about['__version__'],
      classifiers=['Development Status :: 4 - Beta',
                   'Operating System :: POSIX :: Linux',
                   'Programming Language :: Python :: 3',
                   ],
      long_description=open('README.md').read(),
      long_description_content_type='text/markdown',
      url=URL,
      author_email=EMAIL,
      author=AUTHOR,
      license=LICENSE,
      packages=['guietta',
                'guietta.examples',
                ],
      install_requires=['PySide2'],
      test_suite='test',
      cmdclass={'upload': UploadCommand,
                'conda': CondaCommand},
      )



# -*- coding: utf-8 -*-

import os.path
from guietta import Gui, _

# Labels and button automatically load images if the filename is valid,
# otherwise it will appear as text.
#
# Widget names are set to the image name without extension.

gui = Gui(
    
  [  _             , 'up.png'   , _             ],
  [  ['left.png'] ,     _       , ['right.png'] ],
  [  _             , 'down.png' , _             ],
  
    images_dir = os.path.dirname(__file__)

  )

while True:
    name, event = gui.get()

    if name == 'left':
        print('Clicked left!')

    if name == 'right':
        print('Clicked right!')

    if name == None:
        break





# -*- coding: utf-8 -*-

from guietta import Gui, _, ___, III, HValueSlider, VValueSlider, Qt

def do_slider(gui, value):
    print('Slider value: ', value)
    
slider1 = HValueSlider('hvalue', range(500), unit='Hz')    
slider2 = HValueSlider('hvalue', range(500), unit='Hz', anchor=Qt.AnchorLeft) 
slider3 = VValueSlider('hvalue', range(500))    

gui = Gui(
    
  [  'xxx' , ['xxx'], 'xxx' , slider3],
  [  slider1,   ___   , ___ ,   III  ],
  [  slider2,   ___   ,  _  ,   III  ],
  )


gui.events(
    
  [     _     ,    _    ,   _  ,  _  ],
  [  do_slider,    _    ,   _  ,  _  ],
  [     _     ,    _    ,   _  ,  _  ],
   )


gui.run()

    